*! version 1.0.4  20may2010
version 11

local vcerep	`"tokens("bootstrap jackknife")"'
local asmcmds	`"tokens("asmprobit asroprobit asclogit nlogit")"'
local varcov	`"tokens("var cov")"'

mata:

class _b_table {

protected:

	string	scalar	_bmat
	string	scalar	_vmat
	string	scalar	_emat
	real	scalar	_df
	real	scalar	_level
	real	scalar	_legend
	string	scalar	_diopts
	real	scalar	_beta
	real	scalar	_is_mi
	real	scalar	_dftable
	string	scalar	_dfmat
	real	scalar	_dfmis
	string	scalar	_pisemat
	real	scalar	_ndiparm
	string	vector	_diparm

	string	scalar	cformat
	string	scalar	sformat
	string	scalar	pformat

	real	matrix	bmat
	real	matrix	vmat
	real	matrix	emat
	real	matrix	dfmat
	real	matrix	pisemat

	real	scalar	has_bmat
	real	scalar	has_vmat
	real	scalar	has_emat
	real	scalar	has_se

	real	scalar	_z
	string	matrix	stripe
	real	scalar	eq_info
	string	matrix	notes

class	_tab	scalar	Tab

	void		cluster()
	void		compute_multi_line_tsop()
	void		comment()
	void		diparm()

public:

	void		new()
			set_b()
			set_V()
			set_errmat()
			set_df()
			set_level()
			set_legend()
			set_diopts()
			set_beta()
			set_dftable()
			set_dfmat()
			set_pisemat()
			set_ndiparm()
			set_diparm()

			set_cformat()
			set_sformat()
			set_pformat()

	void		validate()

	void		di_titles()
	void		di_eq()
	void		di_comment()
	void		di_offset()
	void		do_diparms()

	void		sep()
	void		finish()

}

// subroutines --------------------------------------------------------------

void _b_table::cluster()
{
	string	scalar	msg
	string	scalar	clustvar
	real	scalar	reps
	string	scalar	cmd
	string	scalar	nclust

	if (_is_mi) {
		msg	= "Within VCE adjusted for"
		reps	= 0
		nclust	= "e(N_clust_mi)"
	}
	else if (any(strmatch(st_global("e(prefix)"), `vcerep'))) {
		msg	= "Replications based on"
		reps	= 1
		nclust	= "e(N_clust)"
	}
	else {
		msg	= "Std. Err. adjusted for"
		reps	= 0
		nclust	= "e(N_clust)"
	}
	clustvar = st_global("e(clustvar)")
	if (strlen(clustvar) == 0 & (reps | st_global("e(vce)") == "robust")) {
		if (_is_mi) cmd	= st_global("e(cmd_mi)")
		else cmd	= st_global("e(cmd)")
		if (substr(cmd,1,2) == "xt") {
			stata(sprintf("is_xt %s", cmd))
			if (st_numscalar("r(is_xt)")) {
				clustvar = st_global("e(ivar)")
			}
		}
		else if (any(strmatch(cmd, tokens("clogit rologit")))) {
			clustvar = st_global("e(group)")
		}
		else if (any(strmatch(cmd, `asmcmds'))) {
			clustvar = st_global("e(case)")
		}
	}
	if (strlen(clustvar)) {
		displayas("txt")
		if (length(st_numscalar(nclust))) {
			printf("{ralign 78:(%s {res:%f} clusters in %s)}\n",
				msg,
				st_numscalar(nclust),
				clustvar)
		}
		else {
			printf("{ralign 78:(%s clustering on %s)}\n",
				msg,
				clustvar)
		}
	}
}

void _b_table::compute_multi_line_tsop(	string	vector	op,
					string	scalar	dv,
					real	scalar	p)
{
	string	scalar	myop, t
	real	scalar	n, i, j

	myop = substr(dv, 1, p-1)

	if (strlen(myop) < 12) {
		op = myop + "."
		dv = abbrev(substr(dv, p+1, .), 12)
		return
	}

	n = ceil(strlen(myop)/12)
	op = J(1,n-1,"")
	j = 1
	for (i=1; i<n; i++) {
		op[i] = substr(myop, j, j+11)
		j = j + 12
	}
	myop = substr(myop, j, .)
	if (strlen(myop) == 0) {
		dv = abbrev(substr(dv, p+1, .), 12)
	}
	else {
		t = myop + "." + substr(dv, p+1, .)
		if (strlen(t) <= 12) {
			swap(dv, t)
		}
		else {
			op = op, (myop + ".")
			dv = abbrev(substr(dv, p+1, .), 12)
		}
	}
}

void _b_table::comment(	string	scalar	id,
			string	scalar	comment,
			real	scalar	header)
{
		vector	hold1
		vector	hold2

	hold1	= Tab.set_color_string()
	if (header) {
		hold2	= Tab.set_format_string()
		Tab.set_color_string(("result","text","","","","",""))
		Tab.set_format_string(("%-12s","","","","","",""))
		Tab.row(J(1,7,.b), (id,comment,"","","","",""))
		Tab.set_format_string(hold2)
	}
	else {
		Tab.set_color_string(("text","text","","","","",""))
		Tab.row(J(1,7,.b), (id,comment,"","","","",""))
	}
	Tab.set_color_string(hold1)
}

void _b_table::diparm(	transmorphic	t,
			string	scalar	extra,
			string	scalar	exp)
{
	real	scalar	rc
	string	scalar	tok
	string	scalar	args
	string	scalar	eqname
	real	scalar	varcov
	string	scalar	label
	real	scalar	len
	real	scalar	eqlab
	real	scalar	b
	real	scalar	se
	real	scalar	z
	real	scalar	p
	real	scalar	lb
	real	scalar	ub
	real	scalar	el
	real	scalar	n, i
	real	scalar	ieq
	real	scalar	pise
	real	scalar	df
	string	scalar	name
	real	scalar	is_base
	real	scalar	prob
	string	scalar	note
		vector	chold
		vector	fhold

	prob	= .
	tok	= tokenget(t)
	if (tok == "__sep__") {
		sep()
	}
	else if (tok == "__bot__") {
		sep("bottom")
	}
	else if (tok == "__lab__") {
		eqlab	= 0
		b	= .b
		label	= ""
		name	= ""
		tok	= tokenget(t)
		len	= strlen(tok)
		if (tok == ",") {
			tok	= tokenget(t)
			len	= strlen(tok)
		}
		else if (len) {
			exit(error(198))
		}
		while (len) {
			if (tok == "label") {
				tok	= tokenget(t)
				len	= strlen(tok)
				if (substr(tok,1,1) != "(") {
					exit(error(198))
				}
				if (substr(tok,len,1) != ")") {
					exit(error(198))
				}
				label	= strtrim(substr(tok,2,len-2))
				if (strmatch(label, `"`"*"'"')) {
					len = strlen(label)
					label	= strtrim(substr(label,3,len-4))
				}
				if (strmatch(label, `""*""')) {
					len = strlen(label)
					label	= strtrim(substr(label,2,len-2))
				}
			}
			else if (tok == substr("eqlabel",1,max((5,len)))) {
				eqlab = 1
			}
			else if (tok == "value") {
				tok	= tokenget(t)
				len	= strlen(tok)
				if (substr(tok,1,1) != "(") {
					exit(error(198))
				}
				if (substr(tok,len,1) != ")") {
					exit(error(198))
				}
				tok	= substr(tok,2,len-2)
				b	= strtoreal(tok)
			}
			else if (tok == "comment") {
				tok	= tokenget(t)
				len	= strlen(tok)
				if (substr(tok,1,1) != "(") {
					exit(error(198))
				}
				if (substr(tok,len,1) != ")") {
					exit(error(198))
				}
				name	= strtrim(substr(tok,2,len-2))
				if (strmatch(name, `"`"*"'"')) {
					len = strlen(name)
					name	= strtrim(substr(name,3,len-4))
				}
				if (strmatch(name, `""*""')) {
					len = strlen(name)
					name	= strtrim(substr(name,2,len-2))
				}
				name	= "   " + name
			}
			else {
				exit(error(198))
			}
			tok	= tokenget(t)
			len	= strlen(tok)
		}
		if (strlen(label) == "") {
			exit(error(198))
		}
		label = abbrev(label,12)
		if (eqlab) {
			chold = Tab.set_color_string()
			fhold = Tab.set_format_string()
			Tab.set_color_string(("result",J(1,6,"")))
			Tab.set_format_string(("%-12s",J(1,6,"")))
		}
		Tab.row((.b,b,.b,.b,.b,.b,.b),
			(label,"",name,"","","",""))
		if (eqlab) {
			Tab.set_color_string(chold)
			Tab.set_format_string(fhold)
		}
	}
	else {
		eqname	= tok
		args	= tok
		tok	= tokenget(t)
		varcov	= 0
		if (any(eqname:==`varcov') & strmatch(tok, "(*)")) {
			varcov	= 1
			eqname	= eqname + tok
			args	= args + tok
			tok	= tokenget(t)
		}
		while (strlen(tok)) {
			if (tok == "label") {
				tok	= tokenget(t)
				len	= strlen(tok)
				if (substr(tok,1,1) != "(") {
					exit(error(198))
				}
				if (substr(tok,len,1) != ")") {
					exit(error(198))
				}
				label	= strtrim(substr(tok,2,len-2))
				if (strmatch(label, `"`"*"'"')) {
					len = strlen(label)
					label	= strtrim(substr(label,3,len-4))
				}
				if (strmatch(label, `""*""')) {
					len = strlen(label)
					label	= strtrim(substr(label,2,len-2))
				}
			}
			else if (any(strmatch(tok, "(*)"))) {
				args = sprintf("%s%s", args, tok)
			}
			else {
				if (tok == "noprob") {
					prob = 0
				}
				else if (tok == "prob") {
					prob = 1
				}
				args = sprintf("%s %s", args, tok)
			}
			tok	= tokenget(t)
		}
		rc = _stata(sprintf(	"_diparm %s %s",
					args,
					extra))
		if (rc) exit(rc)
		b	= st_numscalar("r(est)")
		se	= st_numscalar("r(se)")
		z	= st_numscalar("r(z)")
		p	= st_numscalar("r(p)")
		lb	= st_numscalar("r(lb)")
		ub	= st_numscalar("r(ub)")
		el	= st_numscalar("r(i)")
		if ((missing(prob) & el == 0) | prob == 0) {
			z = .b
			p = .b
		}
		is_base = 0
		if (strlen(label) == 0) {
			label = eqname
			if (varcov) {
				stata(sprintf("_msparse %s:_cons, eq", label))
				label	= st_global("r(stripe)")
				n	= rows(eq_info)
				ieq	= 0
				for (i=1; i<=n; i++) {
					if (label == stripe[eq_info[i,1],1]) {
						ieq = i
						break
					}
				}
				if (ieq == 0) {
					varcov = 0
				}
			}
			if (!varcov) {
				stata(sprintf("_ms_parse_parts %s", label))
				if (st_global("r(type)") == "factor") {
					label = sprintf("%s%s.%s",
						st_numscalar("r(level)"),
						st_global("r(ts_op)"),
						st_global("r(name)"))
					is_base = st_numscalar("r(base)")
				}
				label = "/" + abbrev(label, 11)
			}
		}
		else {
			label = abbrev(label,12)
		}
		if (varcov) {
			stata(sprintf("_ms_eq_display, %s eq(%f) astext",
				_bmat=="e(b)" ? "" : _bmat,
				ieq))
			Tab.set_skip1(1)
		}
		if (b == 0 & (missing(se) | se == 0)) {
			if (is_base) {
				note = notes[6]
			}
			else {
				note = notes[5]
			}
		}
		if (_legend) {
			if (_legend == 2) {
				b = se
			}
			name = ""
			if (el) {
			    if (strlen(stripe[el,1])) {
				name = sprintf(	"  %s[%s:%s]",
						exp,
						stripe[el,1],
						stripe[el,2])
			    }
			    else {
				name = sprintf(	"  %s[%s]",
						exp,
						stripe[el,2])
			    }
			}
			Tab.row((.b,b,.b,.b,.b,.b,.b),
				(label,"",name,"","","",""))
		}
		else if (_beta) {
			Tab.row((.b,b,se,z,p,.b,.),
				(label,"","","","","",""))
		}
		else if (_dftable) {
			df	= .b
			pise	= .b
			if (el) {
				if (el <= cols(dfmat)) {
					df	= dfmat[1,el]
				}
				else {
					df	= .
				}
				if (el <= cols(pisemat)) {
					pise	= pisemat[1,el]
				}
				else {
					pise	= .
				}
			}
			Tab.row((.b,b,se,z,p,df,pise),
				(label,"","","","","",""))
		}
		else {
			if (strlen(note)) {
				b	= .b
				se	= .b
				z	= .b
				p	= .b
				lb	= .b
				ub	= .b
			}
			Tab.row((.b,b,se,z,p,lb,ub),
				(label,note,"","","","",""))
		}
		if (varcov) {
			Tab.set_skip1(.)
		}
	}
}

// callable subroutines -----------------------------------------------------

void _b_table::new()
{
	_df		= .
	_level		= c("level")
	_legend		= 0
	_beta		= 0
	_dftable	= 0
	_dfmis		= 0
	_diparm		= J(0,1,"")
	cformat		= c("cformat")
	sformat		= c("sformat")
	pformat		= c("pformat")
}

function _b_table::set_b(|string scalar bmat)
{
	if (args() == 0) {
		return(_bmat)
	}
	_bmat = bmat
}

function _b_table::set_V(|string scalar vmat)
{
	if (args() == 0) {
		return(_vmat)
	}
	_vmat = vmat
}

function _b_table::set_errmat(|string scalar emat)
{
	if (args() == 0) {
		return(_emat)
	}
	_emat = emat
}

function _b_table::set_df(|real scalar df)
{
	if (args() == 0) {
		return(_df)
	}
	if (missing(df)) {
		_df = .
	}
	else {
		_df = floor(df)
	}
}

function _b_table::set_level(|real scalar level)
{
	if (args() == 0) {
		return(_level)
	}
	if (missing(level)) {
		_level = c("level")
	}
	else {
		if (level < 10 | level > 99.99) {
		    errprintf("level must be between 10 and 99.99 inclusive\n")
		    exit(198)
		}
		_level = level
	}
}

function _b_table::set_legend(|string scalar legend)
{
	if (args() == 0) {
		return(_legend)
	}

	if (legend == "") {
		_legend = 0
	}
	else if (legend == "coeflegend") {
		_legend = 1
	}
	else if (legend == "selegend") {
		_legend = 2
	}
	else {
		errprintf("invalid legend setting\n")
		exit(198)
	}
}

function _b_table::set_diopts(|string scalar diopts)
{
	if (args() == 0) {
		return(_diopts)
	}

	_diopts = diopts
}

function _b_table::set_beta(|real scalar beta)
{
	if (args() == 0) {
		return(_beta)
	}

	if (missing(beta)) {
		_beta = 0
	}
	else {
		_beta = beta != 0
	}
}

function _b_table::set_dftable(|real scalar dftable)
{
	if (args() == 0) {
		return(_dftable)
	}

	if (missing(dftable)) {
		_dftable = 0
	}
	else {
		_dftable = dftable != 0
	}
}

function _b_table::set_dfmat(|string scalar dfmat)
{
	if (args() == 0) {
		return(_dfmat)
	}

	_dfmat = dfmat
}

function _b_table::set_pisemat(|string scalar pisemat)
{
	if (args() == 0) {
		return(_pisemat)
	}

	_pisemat = pisemat
}

function _b_table::set_ndiparm(|real scalar ndiparm)
{
	if (args() == 0) {
		return(_ndiparm)
	}

	if (missing(ndiparm)) {
		_ndiparm = 0
	}
	else {
		if (ndiparm < 0) {
			errprintf("invalid ndiparm setting\n")
			exit(198)
		}
		_ndiparm = floor(ndiparm)
	}
	_diparm = J(_ndiparm,1,"")
}

function _b_table::set_diparm(real scalar i, |string scalar diparm)
{
	if (i > _ndiparm) {
		errprintf("ndiparm setting too small\n")
		exit(198)
	}
	if (args() == 1) {
		return(_diparm[i])
	}

	if (strpos(diparm, ",")) {
		_diparm[i] = diparm
	}
	else {
		_diparm[i] = diparm + ","
	}
}

function _b_table::set_cformat(|string scalar fmt)
{
	if (args() == 0) {
		return(cformat)
	}
	if (strlen(fmt)) {
		cformat = fmt
	}
	else {
		cformat = c("cformat")
	}
}

function _b_table::set_sformat(|string scalar fmt)
{
	if (args() == 0) {
		return(sformat)
	}
	if (strlen(fmt)) {
		sformat = fmt
	}
	else {
		sformat = c("sformat")
	}
}

function _b_table::set_pformat(|string scalar fmt)
{
	if (args() == 0) {
		return(pformat)
	}
	if (strlen(fmt)) {
		pformat = fmt
	}
	else {
		pformat = c("pformat")
	}
}

void _b_table::validate()
{
	string	scalar	n6
	string	scalar	n7

	if (_beta) {
		if (_legend == 1) {
			errprintf("only one of beta or coeflegend can be set\n")
			exit(198)
		}
		else if (_legend == 2) {
			errprintf("only one of beta or selegend can be set\n")
			exit(198)
		}
	}

	_is_mi = st_global("e(mi)") == "mi"
	if (_is_mi) {
		if (_dfmat == "") {
			_dfmat = "e(df_mi)"
		}
		if (_dfmat == "e(df_Q_mi)") {
			_dfmis = st_numscalar("e(_dfnote_Q_mi)")
		}
		else {
			_dfmis = st_numscalar("e(_dfnote_mi)")
		}
		dfmat	= st_matrix(_dfmat)
		if (length(_dfmis) == 0) {
			_dfmis = 0
		}
		if (_pisemat == "") {
			_pisemat = "e(pise_mi)"
		}
	}
	else if (_dftable) {
		errprintf("dftable setting requires mi estimation results\n")
		exit(322)
	}

	if (strlen(cformat) == 0) {
		cformat = "%9.0g"
	}
	if (fmtwidth(cformat) > 9) {
		errprintf("invalid cformat;\nwidth too large\n")
		exit(198)
	}

	if (strlen(sformat) == 0) {
		sformat = "%8.2f"
	}
	if (fmtwidth(sformat) > 8) {
		errprintf("invalid sformat;\nwidth too large\n")
		exit(198)
	}

	if (strlen(pformat) == 0) {
		pformat = "%5.3f"
	}
	if (fmtwidth(pformat) > 5) {
		errprintf("invalid pformat;\nwidth too large\n")
		exit(198)
	}

	if (_dftable) {
		n6 = "%9.1f"
		n7 = "%9.2f"
		pisemat	= st_matrix(_pisemat)
	}
	else {
		n6 = cformat
		n7 = cformat
	}

	if (strlen(_bmat)) {
		bmat = st_matrix(_bmat)
	}
	else {
		_bmat = "e(b)"
	}
	if (strlen(_vmat) == 0) {
		if (st_macroexpand("`"+"e(V)"+"'") == "matrix") {
			_vmat = "e(V)"
		}
	}
	if (strlen(_vmat)) {
		vmat = sqrt(diagonal(st_matrix(_vmat)))'
	}
	if (strlen(_emat)) {
		emat = st_matrix(_emat)
	}

	Tab.init(7)
	Tab.set_lmargin(0)
	Tab.set_width((13,11,11,9,8,13,12))
	Tab.set_vbar((0,1,0,0,0,0,0,0))
	Tab.set_ignore(J(1,7,".b"))
	Tab.set_format_title(("%12s","%11s","%12s","%7s","%8s","%13s","%12s"))
	Tab.set_format_string(("","%11s","%1s","","","",""))
	Tab.set_format_number(("",cformat,cformat,sformat,pformat,n6,n7))
	Tab.set_color_string(("","result","","","","",""))

	_z = (_df > 2e17
	   ? invnormal((100+_level)/200)
	   : invttail(_df, (100-_level)/200))
	stripe	= st_matrixcolstripe(_bmat)
	eq_info = panelsetup(stripe, 1)

	has_bmat = cols(bmat)
	has_vmat = cols(vmat)
	has_emat = cols(emat)
	if (has_vmat | strlen(_vmat)) {
		has_se = 1
	}
	else {
		has_se = 0
	}

	notes	= J(1,8,"")
	notes[1] = "(no observations)"
	notes[2] = "(stratum with 1 PSU detected)"
	notes[3] = "(sum of weights equals zero)"
	notes[4] = "(denominator estimate equals zero)"
	notes[5] = "(omitted)"
	notes[6] = "(base)"
	notes[7] = "(empty)"
	notes[8] = "(not estimable)"
}

void _b_table::di_titles(	string	scalar	depname,
				string	scalar	coefttl,
				string	scalar	cnsreport,
				real	scalar	clreport)
{
	string	scalar	dv
	string	vector	op
	string	scalar	vce
	string	scalar	mse
	string	scalar	vcettl
	real	scalar	vcewd
	string	scalar	vcefmt
	string	scalar	obs
	string	scalar	ttl6
	string	scalar	ttl7
	string	scalar	fmt6
	string	scalar	fmt7
	real	scalar	ciwd
	string	scalar	name
	string	scalar	stat
	real	scalar	p
	real	scalar	plus
	real	scalar	n, i
	real	scalar	rc
		vector	hold
		vector	fhold

	if (strlen(depname) == 0) {
		dv = st_global("e(depvar)")
	}
	else {
		dv = depname
	}
	if (cols(tokens(dv)) > 1) {
		dv = ""
	}
	if (!_is_mi & has_se) {
		vcettl	= st_global("e(vcetype)")
	}
	if (strlen(dv)) {
		stata(sprintf("_msparse %s, ivar", dv))
		dv = st_global("r(stripe)")
		p  = strpos(dv, ".")
		if (p & strlen(dv) > 12) {
			op = J(1,0,"")
			compute_multi_line_tsop(op, dv, p)
		}
		else {
			dv = abbrev(dv, 12)
		}
	}

	if (strlen(coefttl) == 0) {
		if (_legend == 2) {
			coefttl = "Std. Err."
		}
		else {
			coefttl = "Coef."
		}
	}

	if (substr(cnsreport,1,2) != "no") {
		if (date(c("born_date"), "DMY", 2000) < mdy(3,9,2010)) {
errprintf("your Stata executable is out of date\n")
errprintf("    type -update executable- at the Stata prompt\n")
			exit(498)
		}
		stata(sprintf("_makecns, displaycns nullok %s", cnsreport))
	}

	if (clreport) {
		cluster()
	}

	Tab.sep("top")
	n = length(op)
	if (n) {
		hold = Tab.set_format_title()
		Tab.set_format_title(("%-12s","","","","","",""))
		for (i=1; i<n; i++) {
			Tab.titles((op[i],"","","","","",""))
		}
		if (strlen(vcettl) == 0) {
			Tab.titles((op[n],"","","","","",""))
			op = ""
		}
		else {
			op = op[n]
		}
		Tab.set_format_title(hold)
	}
	else {
		op = ""
	}

	if ((_dftable | strlen(vcettl)) & !_legend) {
		vce	= st_global("e(vce)")
		mse	= st_global("e(mse)")
		obs	= ""
		ttl6	= ""
		ttl7	= ""
		fmt7	= ""
		if (_dftable) {
			ttl7 = "% Increase"
			fmt7 = "%12s"
		}
		else if (vce == "bootstrap") {
			obs = "Observed"
			if (vcettl == "Bootstrap" & !_beta) {
				ttl7 = "Normal-based"
			}
		}
		plus	= 0
		vcewd	= strlen(vcettl)
		if (strlen(mse)) {
			name = sprintf("%s_%s", vce, mse)
			stata(sprintf("capture which %s.sthlp", name))
			rc = c("rc")
			if (rc) {
				stata(sprintf("capture which %s.hlp", name))
				rc = c("rc")
			}
			if (!rc) {
				vcettl = sprintf(	"{help %s##|_new:%s}",
							name,
							vcettl)
				plus = strlen(vcettl) - vcewd
			}
			stata("capture")
		}
		vcewd	= vcewd + plus + ceil((12 - vcewd)/2+1)
		vcefmt	= sprintf("%%%fs", vcewd)
		if (!_dftable) {
			ciwd = strlen(ttl7)
			if (ciwd > 2) {
				ciwd = ceil((ciwd-2)/2) - 1
				ttl6 = substr(ttl7, 1, ciwd)
				ttl7 = substr(ttl7, ciwd+1, .)
			}
			fmt7 = "%1s"
		}
		hold = Tab.set_format_title()
		Tab.set_format_title(("%-12s","",vcefmt,"","","",fmt7))
		Tab.titles((op, obs, vcettl, "", "", ttl6, ttl7))
		Tab.set_format_title(hold)
	}

	if (_legend) {
		Tab.set_format_title(("","","%8s","","","",""))
		Tab.titles((	dv,
				coefttl,
				"Legend",
				"",
				"",
				"",
				""))
	}
	else if (has_se) {
		if (_is_mi) {
			stat = "t"
		}
		else {
			stat	= missing(_df) ? "z" : "t"
		}
		if (_dftable) {
			if (_dfmis) {
				ttl6	= "{help mi_missingdf##|_new:DF}"
				fhold	= Tab.set_format_title()
				plus	= fmtwidth(fhold[6]) + strlen(ttl6) - 2
				fmt6	= sprintf("%%%fs", plus)
				Tab.set_format_title((J(1,5,""),fmt6,""))
			}
			else {
				ttl6	= "DF"
			}
			ttl7	= "Std. Err."
		}
		else if (_beta) {
			ttl6	= ""
			ttl7	= "Beta"
		}
		else {
			ttl6	= sprintf("[%g%% Con", _level) 
			ttl7	= "f. Interval]"
		}
		hold	= Tab.set_width()
		Tab.set_width(hold + (0,0,1,-1,0,0,0))
		Tab.titles((	dv,
				coefttl,
				"Std. Err.",
				stat,
				sprintf("P>|%s|", stat),
				ttl6,
				ttl7))
		Tab.set_width(hold)
		if (length(fhold)) {
			Tab.set_format_title(fhold)
		}
	}
	else {
		Tab.titles((	dv,
				coefttl,
				"",
				"",
				"",
				"",
				""))
	}
}

void _b_table::di_eq(	real	scalar	eq,
			real	scalar	hideeq,
			real	scalar	eform,
			string	scalar	comment)
{
	string	scalar	eqname
	real	scalar	i0
	real	scalar	k
	real	scalar	output
	string	scalar	first
	string	scalar	ms_di
	string	scalar	diopts
	real	scalar	i, j
	string	scalar	matopt
	real	scalar	census
	real	scalar	b
	real	scalar	se
	real	scalar	beta
	real	scalar	t
	real	scalar	z
	real	scalar	df
	real	scalar	pval
	real	scalar	ll
	real	scalar	ul
	real	scalar	pise
	real	scalar	err
	string	scalar	exp
	string	scalar	name
	string	scalar	note
	string	vector	nfmt
	real	scalar	has_eq
		vector	nhold
		vector	shold

	if (eq < 1 | eq > rows(eq_info)) {
		exit(error(303))
	}

	census	= 0
	if (st_global("e(prefix)") == "svy") {
		if (length(st_numscalar("e(census)"))) {
			census = st_numscalar("e(census)")
		}
	}

	Tab.sep()

	has_eq	= rows(eq_info) > 1
	i0	= eq_info[eq,1]
	k	= eq_info[eq,2] - i0 + 1
	eqname	= stripe[i0,1]
	if (strlen(comment)) {
		di_comment(eqname, comment)
		return
	}

	if (_legend == 1) {
		exp = "_b"
	}
	else {
		exp = "_se"
	}

	if (eqname == "_" | hideeq) {
		eqname = ""
	}
	matopt	= sprintf("mat(%s)", _bmat)
	if (strlen(eqname)) {
		stata(sprintf("_ms_eq_display, eq(%f) %s", eq, matopt))
		printf("\n")
	}
	output	= 0
	first	= ""
	diopts	= _diopts
	if (!strpos(diopts, "vsquish")) {
		diopts = diopts + " vsquish"
	}
	if (eform) {
		ms_di	= "_ms_display, eq(#%f) el(%f) nocons %s %s %s"
	}
	else {
		ms_di	= "_ms_display, eq(#%f) el(%f) %s %s %s"
	}
	nfmt	= J(1,7,"")
	nhold	= Tab.set_format_number()
	shold	= Tab.set_format_string()
	Tab.set_format_string(("","%-11s","","","","",""))
	Tab.set_skip1(1)
	z	= _z
	df	= _df
	j	= i0
	for (i=1; i<=k; i++,j++) {
		stata(sprintf(ms_di, eq, i, first, diopts, matopt))
		if (st_numscalar("r(output)")) {
			first = ""
			if (!output) {
				output = 1
				diopts = _diopts
			}
		}
		else {
			if (st_numscalar("r(first)")) {
				first = "first"
			}
			continue
		}
		note	= st_global("r(note)")

		if (has_bmat) {
			b	= bmat[1,j]
			if (has_vmat) {
				se	= vmat[1,j]
			}
			else {
				se	= 0
			}
		}
		else {
			b	= st_numscalar("r(b)")
			se	= st_numscalar("r(se)")
			if (_beta) {
				if (length(st_numscalar("r(beta)"))) {
					beta = st_numscalar("r(beta)")
				}
			}
		}

		if (_is_mi) {
			if (j <= cols(dfmat)) {
				df	= dfmat[1,j]
			}
			else {
				df	= .
			}
		}
		if (_dftable) {
			if (j <= cols(pisemat)) {
				pise	= pisemat[1,j]
			}
			else {
				pise	= .
			}
		}
		else {
			if (_is_mi) {
				z = (df > 2e17
				  ? invnormal((100+_level)/200)
				  : invttail(df, (100-_level)/200))
			}
		}
		if (has_se & se == 0 & census) {
			t	= .
			pval	= .
			ll	= .
			ul	= .
		}
		else {
			if (se == 0) {
				se = .
			}
			t	= b/se
			if (df > 2e17) {
				pval = 2*normal(-abs(t))
			}
			else {
				pval = 2*ttail(df,abs(t))
			}
			ll = b - se*z
			ul = b + se*z
		}

		if (b == 0) {
			if (has_se & missing(se)) {
				if (strlen(note) == 0) {
					note = notes[5]
				}
			}
		}
		else {
			note = ""
		}
		if (has_emat) {
			err	= emat[1,j]
			if (err >=1 & err <= 8 & floor(err) == err) {
				note = notes[err]
			}
		}
		if (strlen(note)) {
			note = "  " + note
		}
		if (eform) {
			b	= exp(b)
			se	= se*b
			if (!_dftable) {
				ll	= exp(ll)
				ul	= exp(ul)
			}
		}
		if (_legend) {
			if (_legend == 2) {
				b = se
			}
			if (has_eq & strlen(stripe[j,1])) {
				name = sprintf(	"  %s[%s:%s]",
						exp,
						stripe[j,1],
						stripe[j,2])
			}
			else {
				name = sprintf(	"  %s[%s]",
						exp,
						stripe[j,2])
			}
			if (strlen(note)) {
				Tab.row((.b,.b,.b,.b,.b,.b,.b),
					("",note,name,"","","",""))
			}
			else {
				Tab.row((.b,b,.b,.b,.b,.b,.b),
					("","",name,"","","",""))
			}
		}
		else if (strlen(note)) {
			Tab.row((.b,.b,.b,.b,.b,.b,.b),
				("",note,"","","","",""))
		}
		else {
			if (!missing(t) & abs(t) > 9999) {
				nfmt[4] = "%8.2e"
			}
			else {
				nfmt[4] = nhold[4]
			}
			Tab.set_format_number(nfmt)
			if (!has_se) {
				Tab.row((.b,b,J(1,5,.b)))
			}
			else if (_beta) {
				Tab.row((.b,b,se,t,pval,.b,beta))
			}
			else if (_dftable) {
				Tab.row((.b,b,se,t,pval,df,pise))
			}
			else {
				Tab.row((.b,b,se,t,pval,ll,ul))
			}
		}
	}
	Tab.set_format_number(nhold)
	Tab.set_format_string(shold)
	Tab.set_skip1(.)
}

void _b_table::di_comment(string vector idlist, string scalar comment)
{
	real	scalar	k
	real	scalar	i

	k = length(idlist)
	if (k==1 & strlen(idlist) == 0) {
		comment("", comment, 1)
	}
	else {
		if (k == 1) {
			comment(idlist[1], comment, 1)
		}
		else {
			comment(idlist[1], "", 0)
		}
		for (i=2; i<=k; i++) {
			comment(idlist[i], comment, 0)
		}
	}
}

void _b_table::di_offset(string scalar offset)
{
	real	scalar	len
	string	scalar	name
	string	scalar	type
		vector	hold

	len	= strlen(offset)
	if (len == 0) {
		return
	}
	if (substr(offset,1,3) == "ln(") {
		name = substr(offset,4,len-4)
		type = "(exposure)"
	}
	else {
		name = offset
		type = "(offset)"
	}

	hold	= Tab.set_color_string()
	Tab.set_color_string(("","text","","","","",""))
	Tab.row(J(1,7,.b), (name,type,"","","","",""))
	Tab.set_color_string(hold)
}

void _b_table::do_diparms()
{
	real	scalar	i
	string	scalar	extra
	transmorphic	t
	string	scalar	exp

	if (_ndiparm == 0) {
		return
	}

	t = tokeninit(" ", ",", (`""""', `"`""'"', char(96)+char(39), "()"))

	if (_legend == 1) {
		exp	= "_b"
	}
	else if (_legend == 2) {
		exp	= "_se"
	}

	extra	= sprintf("level(%g) notab bmat(%s) vmat(%s) dfmat(%s)",
				_level,
				_bmat == "e(b)" ? "" : _bmat,
				_vmat == "e(V)" ? "" : _vmat,
				_vmat == "e(V)" ? "" : _dfmat)

	sep()
	for (i=1; i<=_ndiparm; i++) {
		tokenset(t, _diparm[i])
		diparm(t, extra, exp)
	}
}

void _b_table::sep(|string scalar type)
{
	if (args() == 0) {
		type = "middle"
	}
	Tab.sep(type)
}

void _b_table::finish()
{
	Tab.sep("bottom")
}

end
