{smcl}
{* *! version 1.1.1  29jul2010}{...}
{cmd:help cluster}
{hline}

{title:Title}

{p2colset 5 21 23 2}{...}
{p2col :{manlink MV cluster} {hline 2}}Introduction to cluster-analysis commands{p_end}
{p2colreset}{...}


{title:Syntax}

    Cluster analysis of data

	{cmd:cluster} {it:subcommand} {it:...}

 
    Cluster analysis of a dissimilarity matrix

	{cmd:clustermat} {it:subcommand} {it:...}


{title:Description}

{pstd}
Stata's cluster-analysis routines provide several hierarchical and
partition clustering methods, postclustering summarization methods, and
cluster-management tools.  This entry presents an overview of
the {cmd:cluster} and {cmd:clustermat} commands (also see
{manhelp clustermat MV}), as well as Stata's cluster-analysis management
tools.  The hierarchical clustering methods may be applied
to the data by using the {cmd:cluster} command or to a user-supplied
dissimilarity matrix by using the {cmd:clustermat} command.

{pstd}
The {cmd:cluster} command has the following {it:subcommands}, which
are detailed in their respective manual entries.

	Topic{col 44}{cmd:cluster} {it:subcommand}
	{hline 65}
	Partition-clustering methods for observations
	(see {manhelp cluster_kmeans MV:cluster kmeans and kmedians})
	    Kmeans{col 44}{cmd:cluster kmeans}
	    Kmedians{col 44}{cmd:cluster kmedians}

	Hierarchical clustering methods for observations
	(see {manhelp cluster_linkage MV:cluster linkage})
	    Single linkage{col 44}{cmd:cluster singlelinkage}
	    Average linkage{col 44}{cmd:cluster averagelinkage}
	    Complete linkage{col 44}{cmd:cluster completelinkage}
	    Weighted-average linkage{col 44}{cmd:cluster waveragelinkage}
	    Median linkage{col 44}{cmd:cluster medianlinkage}
	    Centroid linkage{col 44}{cmd:cluster centroidlinkage}
	    Ward's linkage{col 44}{cmd:cluster wardslinkage}

	Postclustering commands
	    Stopping rules{col 44}{helpb cluster stop}
 	    Dendrograms (cluster trees){col 44}{helpb cluster dendrogram}
		{col 44}(synonym: {cmd:cluster} {cmdab:t:ree})
	    Generate summary variables{col 44}{helpb cluster generate}

	User utilities
	    Cluster notes{col 44}{helpb cluster notes}
	    Other user utilities
	    (see {manhelp cluster_utility MV:cluster utility})
		{col 44}{cmd:cluster dir}
		{col 44}{cmd:cluster list}
		{col 44}{cmd:cluster drop}
		{col 44}{cmd:cluster use}
		{col 44}{cmd:cluster rename}
		{col 44}{cmd:cluster renamevar}

	Programmer utilities
	(see {manhelp cluster_programming MV:cluster programming})
		{col 44}{cmd:cluster query}
		{col 44}{cmd:cluster set}
		{col 44}{cmd:cluster} {cmdab:del:ete}
		{col 44}{cmd:cluster} {cmdab:parsedist:ance}
		{col 44}{cmd:cluster measures}

	(Dis)similarity measures
	(see {manhelpi measure_option MV})
	{hline 65}

	{pstd}
	The {cmd:clustermat} command has the following {it:subcommands},
	which are detailed along with the related {cmd:cluster} command
	in the {help cluster linkage} help file.  Also see
	{manhelp clustermat MV}.

	Topic{col 44}{cmd:clustermat} {it:subcommand}
	{hline 65}
	Hierarchical clustering of a dissimilarity matrix
	(see {manhelp cluster_linkage MV:cluster linkage})
	    Single linkage{col 44}{cmd:clustermat singlelinkage}
	    Complete linkage{col 44}{cmd:clustermat completelinkage}
	    Average linkage{col 44}{cmd:clustermat averagelinkage}
	    Weighted average linkage{col 44}{cmd:clustermat waveragelinkage}
	    Median linkage{col 44}{cmd:clustermat medianlinkage}
	    Centroid linkage{col 44}{cmd:clustermat centroidlinkage}
	    Ward's linkage{col 44}{cmd:clustermat wardslinkage}
	{hline 65}


{title:Partition-clustering methods for observations}

{right:(see {hi:[MV] cluster kmeans and kmedians}; help {helpb cluster kmeans and kmedians})  }

    Kmeans cluster analysis

INCLUDE help cluster_kmeans_syntax


    Kmedians cluster analysis

INCLUDE help cluster_kmedians_syntax


INCLUDE help cluster_kmeans_optstab


{title:Hierarchical clustering for observations}

{right:(see {hi:[MV] cluster linkage}; help {help cluster linkage})  }

{p 8 30 2}
{cmd:cluster} {it:linkage} [{varlist}] {ifin}
[{cmd:,} {it:options} ]


INCLUDE help clus_linkage

INCLUDE help cluster_hier_optstab


{title:Postclustering commands}

     Stopping rules{...}
{right:(see {hi:[MV] cluster stop};  }
{right:help {helpb cluster stop})  }

{pstd}
Cluster stopping rules for hierarchical clustering are provided with the
{cmd:cluster stop} command.

INCLUDE help cluster_stop_syntax


    Dendrograms{...}
{right:(see {hi:[MV] cluster dendrogram};  }
{right:help {helpb cluster dendrogram})  }

INCLUDE help cluster_dendro_optstab


    Generate summary variables{...}
{right:(see {hi:[MV] cluster generate};  }
{right:help {helpb cluster generate})  }

{pstd}
The {cmd:cluster generate} command generates summary or grouping variables
after a cluster analysis.  The {cmd:groups()} function generates variables
indicating cluster membership into the specified number(s) of clusters after a
hierarchical cluster analysis.  The {cmd:cut()} function generates a variable
indicating cluster membership based on cutting the dendrogram at the specified
(dis)similarity value.


INCLUDE help cluster_generate_optstab


{title:User utilities}

    Cluster notes{...}
{right:(see {hi:[MV] cluster notes};  }
{right:help {helpb cluster notes})  }

{pstd}
The {cmd:cluster notes} command provides the ability to add, view, and
delete notes for a cluster analysis.


INCLUDE help cluster_notes_syntax


    User utilities{...}
{right:(see {hi:[MV] cluster utility};  }
{right:help {help cluster utility})  }

{pstd}
{cmd:cluster} user utility subcommands allow you to view and manipulate
cluster objects.  {cmd:cluster dir} gives a directory-style listing of the
currently defined clusters.  {cmd:cluster list} gives a detailed
listing of clusters.  {cmd:cluster drop} removes the named
clusters.  {cmd:cluster use} marks a cluster analysis as the most recent one.
{cmd:cluster rename} allows the renaming of a cluster analysis.
{cmd:cluster renamevar} properly renames variables attached to a cluster
analysis.


INCLUDE help cluster_utility_syntax


{title:Programmer utilities}{...}
{right:(see {hi:[MV] cluster programming utilities};  }
{right:help {help cluster programming})            }

{pstd}The {cmd:query}, {cmd:set}, and {cmd:delete} subcommands of {cmd:cluster}
provide programmers a method of obtaining, setting, and deleting the
underlying information and structures of a cluster analysis.  The
{cmd:parsedistance} subcommand provides parsing of distance options for
programmers.  The {cmd:measures} subcommand computes (dis)similarities.


INCLUDE help cluster_programming_syntax


    (Dis)similarity measures{...}
{right:(see {hi:[MV]} {it:measure_option};  }
{right:help {it:{help measure_option}})  }

{pstd}
Measures are divided into those for continuous data and binary data.
{it:measure} capitalization does not matter.  Full definitions are presented
in the {hi:Continuous measure definitions} and {hi:Binary measure definitions}
sections.

INCLUDE help measure_option_optstab


{title:Hierarchical clustering of a dissimilarity matrix}

{right:(see {hi:[MV] cluster linkage}; help {help cluster linkage})  }

{p 8 30 2}
{cmd:clustermat} {it:linkage} [{varlist}] {ifin}
[{cmd:,} {it:clustermat_opts} ]

INCLUDE help clus_linkage

INCLUDE help clustermat_hier_optstab


{title:Also see}

{psee}
Manual:  {manlink MV cluster}

{psee}
{space 2}Help:  {manhelp clustermat MV},
{manhelp cluster_programming MV:cluster programming utilities}
{p_end}
