{smcl}
{* *! version 1.2.4  08jun2010}{...}
{cmd:help stata/ic}{right:{browse "http://www.stata.com"}}
{hline}

{title:Using Stata/IC}

{pstd}
There are four flavors of Stata:

{col 13}{it:flavor}{col 29}{it:description}
{col 13}{hline 47}
{col 13}{bf:Small Stata}{col 29}for undergraduate students
{col 10}-> {bf:Stata/IC}{col 29}standard version 
{col 13}{bf:Stata/SE}{col 29}Stata/IC + large datasets
{col 13}{bf:Stata/MP}{col 29}Stata/SE + parallel processing
{col 13}{hline 47}
{col 13}See {bf:{help flavors:[U] 5 Flavors of Stata}} for descriptions

{pstd}
To determine which flavor of Stata you are running, type

	    {cmd:. about}

{pstd}
If you are using a different flavor of Stata, click on the appropriate
link:  

{col 13}{hline 47}
{col 13}{bf:{help small_stata:Small Stata}}{col 29}Using Small Stata
{col 13}{bf:{help statase:Stata/SE}}{col 29}Using Stata/SE
{col 13}{bf:{help statamp:Stata/MP}}{col 29}Using Stata/MP
{col 13}{hline 47}

{pstd}
For information on upgrading to Stata/IC, Stata/SE or Stata/MP, point your
browser to {browse "http://www.stata.com"}.


{title:Contents}

	{help stataic##starting:1.  Starting Stata/IC}

	{help stataic##setting:2.  Setting Stata/IC's limits}
	    {help stataic##matsize:2.1  Advice on setting matsize}
	    {help stataic##memory:2.2  Advice on setting memory}

	{help stataic##dta:3.  Sharing .dta datasets with non-IC users}

	{help stataic##query:4.  Querying memory usage}

	{help stataic##programming:5.  Advice to programmers:  Determining flavor}



{marker starting}{...}
{title:1.  Starting Stata/IC}

{pstd}
You start Stata/IC as follows:

{p 8 12 4}
    Windows:{break}
	Select {bf:Start > Programs > Stata > Stata {ccl stata_version}}

{p 8 12 4}
    Mac:{break}
        Double-click the file {hi:Stata.do} from the {hi:data} folder, or
        double-click the {hi:Stata} icon from the {hi:Stata} folder.

{p 8 12 4}
    Unix:{break}
	At the Unix command prompt, type {cmd:xstata} to invoke the
	GUI version of Stata/IC, or type {cmd:stata} to invoke the
	console version.


{marker setting}{...}
{title:2.  Setting Stata/IC's limits}

{pstd}
The three limits for Stata/IC are as follows:

{p 8 16 4}
    1.  {cmd:maxvar}{break}
	    The maximum number of variables allowed in a dataset.
	    This limit is set to 2,047 and cannot be reset.

{p 8 16 4}
   2.  {cmd:matsize}{break}
	    The maximum size of matrices, or said differently, the
	    maximum number of independent variables allowed in the models that
	    you fit.  This limit is initially set to 400, and you can
	    increase it up to 800.

{p 8 16 4}
    3.  {cmd:memory}{break}
	    The amount of memory Stata requests from the operating system
	    to store your data.  This limit is initially set to
	    10 MB.  You may set it to as large a number as your
	    operating system will allow.

{pstd}
You set the limits by using the

	    {cmd:set matsize}   {it:#}          [{cmd:,} {cmdab:perm:anently}]
	    {cmd:set memory}    {it:#}[{cmd:b}|{cmd:k}|{cmd:m}|{cmd:g}] [{cmd:,} {cmdab:perm:anently}]

{pstd}
commands.  For instance, you might type

	    {cmd:. set matsize 500}
	    {cmd:. set memory  50m}

{pstd}
The order in which you set the limits does not matter.  If you specify the
{cmd:permanently} option when you set a limit, in addition to making the
change for the present session, Stata/IC will remember the new limit and use
it in the future when you invoke Stata/IC:

	    {cmd:. set matsize 500, permanently}
	    {cmd:. set memory  50m, permanently}

{pstd}
You can reset the present or permanent limits whenever and as often as you
wish.


{marker matsize}{...}
{title:2.1  Advice on setting matsize}

	    {cmd:set matsize} {it:#} [{cmd:,} {cmdab:perm:anently}]{right:10 <= {it:#} <= 800       }


{pstd}
The name {cmd:matsize} is unfortunate because it suggests something that is
only partially true.  It suggests that the maximum size of matrices is
{cmd:matsize} {it:x} {cmd:matsize}.  {cmd:matsize}, however, is 
irrelevant for the size of matrices in Mata, Stata's modern matrix-programming
language.  Regardless of the value of {cmd:matsize}, Mata
matrices be larger or smaller than that.

{pstd}
{cmd:matsize} specifies the maximum size of matrices in Stata's old matrix
language -- and that is not of great importance -- and it specifies the
maximum number of variables that may appear in Stata's estimation commands --
and that is important.  A better name for {cmd:matsize} would be
{cmd:modelsize}.

{pstd}
With that introduction, let us begin.

{pstd}
The value of
{cmd:matsize} has a dramatic effect on memory usage, the formula being

{center:Number of megabytes = (8*{cmd:matsize}^2 + 88*{cmd:matsize})/(1024^2)}

{pstd}
For instance, 

{center:{c TLC}{hline 11}{c TT}{hline 14}{c TRC}}
{center:{c |}  {cmd:matsize}  {c |}  memory use  {c |}}
{center:{c LT}{hline 11}{c +}{hline 14}{c RT}}
{center:{c |}       40  {c |}       .015M  {c |}}
{center:{c |}      100  {c |}       .084M  {c |}}
{center:{c |}      200  {c |}       .322M  {c |}}
{center:{c |}      400  {c |}      1.254M  {c |}}
{center:{c |}      800  {c |}      4.950M  {c |}}
{center:{c BLC}{hline 11}{c BT}{hline 14}{c BRC}}

{pstd}
The formula, in fact, understates the amount of memory certain
Stata commands use and understates what you will use
yourself if you use Stata's old matrix language matrices directly.  The
formula gives the amount of memory required for one matrix and 11 vectors.  If
two matrices are required, the numbers above are nearly doubled.

{pstd}
Choose a reasonable value given the kinds of models
you fit.


{marker memory}{...}
{title:2.2  Advice on setting memory}

	{cmd:set memory} {it:#} [{cmd:b}|{cmd:k}|{cmd:m}|{cmd:g}] [{cmd:,} {cmdab:perm:anently}]{right:{it:#} >= 500k    }

{pin2}
({cmd:b}, {cmd:k}, {cmd:m}, and {cmd:g} may be typed in uppercase.)


{pstd}
You should set enough memory that your datasets fit easily, but not so much that you exceed the physical memory present on your computer, except in emergencies.

{pstd}
You may set memory in bytes (b), kilobytes (k), megabytes (m), or gigabytes
(g), but the number specified must be an integer, so if you want to set 1.5g,
you must set 1,500m.  Actually, 1.5g is 1,536m because the formulas for a
kilobyte, megabyte, and gigabyte are

		1,024 bytes = 1 KB
		1,024 KB    = 1 MB
		1,024 MB    = 1 GB

{pstd}
This detail does not matter, but this is the rule that Stata uses when
presenting numbers, so do not be surprised when 2,000k is not displayed
as 2M or 2M is displayed as 2,048k.

{pstd}
If you have a 32-bit computer, the maximum amount of memory you can
set may surprise you.  Many people think that 32-bit computers can allow up to
4 GB of memory, and that in a sense is true.  Some 32-bit computers
will even allow you to install 4 GB of physical memory.  Nevertheless,
most modern operating systems will allocate a maximum of one-half the
theoretical maximum to individual tasks, meaning that most operating
systems will allow Stata only 2 GB of memory (even if they have 4 GB of
memory).

{pstd}
The same one-half rule applies to 64-bit computers, but half the theoretical
limit is still 536,870,912 GB, so no one much cares.

{pstd}
This one-half limit is imposed by the operating system, not by Stata, and
the operating-system developers have good technical reasons for imposing
the rule.


{marker dta}{...}
{title:3.  Sharing .dta datasets with non-IC users}

{pstd}
You may share datasets with Stata/SE and Stata/MP users with no changes 
necessary.

{pstd}
You may share datasets with Small Stata users as long as your dataset does not
have more variables than are allowed in Small Stata; see {help limits}.


{marker query}{...}
{title:4.  Querying memory usage}

{pstd}
The command

	    {cmd:. query memory}

{pstd}
will display the memory report that {cmd:set maxvar}, {cmd:set matsize},
and {cmd:set memory} display.  This way you do not have to change a value
to find out what you are using and how much memory is consumed by the
limits that you set.


{marker programming}{...}
{title:5.  Advice to programmers:  Determining flavor}

{pstd}
Programmers can determine which flavor of Stata is running by 
examining the {help creturn} values

		                 creturn values
                        {c |} {cmd:c(flavor)   c(SE)     c(MP)}
	    {hline 12}{c +}{hline 30}
	    Small Stata {c |}  "{cmd:Small}"      0         0
	    Stata/IC    {c |}  "{cmd:IC}"         0         0
	    Stata/SE    {c |}  "{cmd:IC}"         1         0
	    Stata/MP    {c |}  "{cmd:IC}"         1         1
	    {hline 12}{c BT}{hline 30}


{title:Also see}

{psee}
{space 2}Help:  
{manhelp about R},
{manhelp data_types D:data types},
{manhelp matsize R},
{manhelp memory D},
{manhelp set R},
{manhelp flavors U:Flavors of Stata},
{help limits},
{help small_stata:Small Stata},
{help statamp:Stata/MP},
{help statase:Stata/SE}
{p_end}
