{smcl}
{* *! version 1.2.4  08jun2010}{...}
{cmd:help stata/se}{right:{browse "http://www.stata.com"}}
{hline}

{title:Using Stata/SE}

{pstd}
There are four flavors of Stata:

{col 13}{it:flavor}{col 29}{it:description}
{col 13}{hline 47}
{col 13}{bf:Small Stata}{col 29}for undergraduate students
{col 13}{bf:Stata/IC}{col 29}standard version 
{col 10}-> {bf:Stata/SE}{col 29}Stata/IC + large datasets
{col 13}{bf:Stata/MP}{col 29}Stata/SE + parallel processing
{col 13}{hline 47}
{col 13}See {bf:{help flavors:[U] 5 Flavors of Stata}} for descriptions

{pstd}
To determine which flavor of Stata you are running, type

	    {cmd:. about}

{pstd}
If you are using a different flavor of Stata, click on the appropriate
link:  

{col 13}{hline 47}
{col 13}{bf:{help small_stata:Small Stata}}{col 29}Using Small Stata
{col 13}{bf:{help stataic:Stata/IC}}{col 29}Using Stata/IC
{col 13}{bf:{help statamp:Stata/MP}}{col 29}Using Stata/MP
{col 13}{hline 47}

{pstd}
For information on upgrading to Stata/SE or Stata/MP, point your browser to
{browse "http://www.stata.com"}.


{title:Contents}

	1.  {help statase##starting:Starting Stata/SE}

	2.  {help statase##setting:Setting Stata/SE's limits}
	    2.1  {help statase##maxvar:Advice on setting maxvar}
	    2.2  {help statase##matsize:Advice on setting matsize}
	    2.3  {help statase##memory:Advice on setting memory}

	3.  {help statase##dta:Sharing .dta datasets with non-SE users}

	4.  {help statase##query:Querying memory usage}

	5.  {help statase##programmers:Advice to programmers}
	    5.1  {help statase##flavor:Determining flavor}
	    5.2  {help statase##macshift:Avoid macro shift in program loops}


{marker starting}{...}
{title:1.  Starting Stata/SE}

{pstd}
You start Stata/SE in much the same way as you start Stata/IC:

{p 8 12 4}
    Windows:{break}
	Select {bf:Start > Programs > Stata > StataSE {ccl stata_version}}

{p 8 12 4}
    Mac:{break}
        Double-click the file {hi:Stata.do} from the {hi:data} folder, or
        double-click the {hi:StataSE} icon from the {hi:Stata} folder.

{p 8 12 4}
    Unix:{break}
	At the Unix command prompt, type {cmd:xstata-se} to invoke the
	GUI version of Stata/SE, or type {cmd:stata-se} to invoke the
	console version.


{marker setting}{...}
{title:2.  Setting Stata/SE's limits}

{pstd}
The three limits for Stata/SE are as follows:

{p 8 16 4}
    1.  {cmd:maxvar}{break}
	    The maximum number of variables allowed in a dataset.
	    This limit is initially set to 5,000; you can increase it
	    up to 32,767.

{p 8 16 4}
    2.  {cmd:matsize}{break}
	    The maximum size of matrices, or said differently, the
	    maximum number of independent variables allowed in the models that
	    you fit.  This limit is initially set to 400, and you can
	    increase it up to 11,000.

{p 8 16 4}
    3.  {cmd:memory}{break}
	    The amount of memory Stata requests from the operating system
	    to store your data.  This limit is initially set to
	    50 MB.  You may set it to as large a number as your
	    operating system will allow.

{pstd}
You reset the limits by using the

	    {cmd:set maxvar}  {it:#}          [{cmd:,} {cmdab:perm:anently}]
	    {cmd:set matsize} {it:#}          [{cmd:,} {cmdab:perm:anently}]
	    {cmd:set memory}  {it:#}[{cmd:b}|{cmd:k}|{cmd:m}|{cmd:g}] [{cmd:,} {cmdab:perm:anently}]

{pstd}
commands.  For instance, you might type

	    {cmd:. set maxvar  6000}
	    {cmd:. set matsize 900}
	    {cmd:. set memory  100m}

{pstd}
The order in which you set the limits does not matter.  If you specify the
{cmd:permanently} option when you set a limit, in addition to making the
change for the present session, Stata/SE will remember the new limit and use
it in the future when you invoke Stata/SE:

	    {cmd:. set maxvar  6000, permanently}
	    {cmd:. set matsize 900, permanently}
	    {cmd:. set memory  50m, permanently}

{pstd}
You can reset the current or permanent limits whenever and as often as you
wish.


{marker maxvar}{...}
{title:2.1  Advice on setting maxvar}

	    {cmd:set maxvar}  {it:#} [{cmd:,} {cmdab:perm:anently}]{right:2,048 <= {it:#} <= 32,767    }


{pstd}
Why is there a limit on {cmd:maxvar}?  Why not just set {cmd:maxvar} to
32,767 and be done with it?  Because simply allowing room for variables, even
if they do not exist, consumes memory, and if you will be
using only datasets with a lot fewer variables, you will be wasting memory.

{pstd}
The formula for the amount of memory consumed by {cmd:set maxvar} is
approximately

{center:Number of megabytes = .3147*({cmd:maxvar}/1000) + .002  }

{pstd}
For instance, if you set {cmd:maxvar} to 20,000, the memory used would be
approximately

{center:Number of megabytes = .3147*20 + .002 = 6.296 MB}

{pstd}
and if you left it at the default, the memory use would be roughly

{center:Number of megabytes = .3147*5  + .002 = 1.575 MB}

{pstd}
Thus how big you set {cmd:maxvar} does not dramatically affect memory usage.
Still, at {cmd:maxvar}=32,000, memory use is 10.072 MB.

{p 8 8 4}
    {bf:Recommendation}:  Think about datasets with the most variables that
    you typically use.  Set {cmd:maxvar} to a few hundred or even 1,000
    above that.  (The memory cost of an extra 1,000 variables is
    only .317 MB.)

{p 8 8 4}
    {bf:Remember}, you can always reset {cmd:maxvar} temporarily by typing
    {cmd:set maxvar} {it:#}.

{p 8 8 4}
    {it:Technical note:}
    The formula above is only approximate.
    When you set {cmd:maxvar},
    Stata/SE will give you a memory report showing the exact amount of
    memory used:

	{hline 68}
	{cmd}. set maxvar 10000
	{txt}
	{title:Current memory allocation}

{col 25}current{col 65}memory usage
{col 9}settable{col 27}value{col 37}description{col 65}(1M = 1024k)
{col 9}{hline 68}
{col 9}set maxvar{col 26}{res: 10000}{col 37}max. variables allowed           3.893M
{col 9}set memory          {res:50M}{col 37}max. data space                 50.000M
{col 9}set matsize{col 25}{res:    400}{col 37}max. RHS vars in models          1.254M
{col 65}{hline 11}
{col 59}          55.147M{txt}
	{hline 68}


{marker matsize}{...}
{title:2.2  Advice on setting matsize}

	    {cmd:set matsize} {it:#} [{cmd:,} {cmdab:perm:anently}]{right:10 <= {it:#} <= 11,000    }


{pstd}
The name {cmd:matsize} is unfortunate because it suggests something that is
only partially true.  It suggests that the maximum size of matrices is
{cmd:matsize} {it:x} {cmd:matsize}.  {cmd:matsize}, however, is 
irrelevant for the size of matrices in Mata, Stata's modern matrix-programming
language.  Regardless of the value of {cmd:matsize}, Mata
matrices be larger or smaller than that.

{pstd}
{cmd:matsize} specifies the maximum size of matrices in Stata's old matrix
language -- and that is not of great importance -- and it specifies the
maximum number of variables that may appear in Stata's estimation commands --
and that is important.  A better name for {cmd:matsize} would be
{cmd:modelsize}.

{pstd}
With that introduction, let us begin.

{pstd}
Although {cmd:matsize} can theoretically be set up to 11,000, on all but the
largest 64-bit computers you will be unable to do that, and even if you
succeeded, Stata/SE would probably run out of memory.  The value of
{cmd:matsize} has a dramatic effect on memory usage, the formula being

{center:Number of megabytes = (8*{cmd:matsize}^2 + 88*{cmd:matsize})/(1024^2)}

{pstd}
For instance, 

{center:{c TLC}{hline 11}{c TT}{hline 14}{c TRC}}
{center:{c |}  {cmd:matsize}  {c |}  memory use  {c |}}
{center:{c LT}{hline 11}{c +}{hline 14}{c RT}}
{center:{c |}      400  {c |}      1.254M  {c |}}
{center:{c |}      800  {c |}      4.950M  {c |}}
{center:{c |}    1,600  {c |}     19.666M  {c |}}
{center:{c |}    3,200  {c |}     78.394M  {c |}}
{center:{c |}    6,400  {c |}    313.037M  {c |}}
{center:{c |}   11,000  {c |}    924.080M  {c |}}
{center:{c BLC}{hline 11}{c BT}{hline 14}{c BRC}}

{pstd}
The formula, in fact, understates the amount of memory certain
Stata commands use and understates what you will use
yourself if you use Stata's old matrix language matrices directly.  The
formula gives the amount of memory required for one matrix and 11 vectors.  If
two matrices are required, the numbers above are nearly doubled.  When you
{cmd:set matsize}, Stata will refuse if you specify too large a value, but
remember that just because Stata does not complain, you still may run into
problems later.  Stata might be running some statistical command and then
complain, "op. sys. refuses to provide memory; r(909)".

{pstd}
For {cmd:matsize}=11,000, nearly 1 GB of memory is required, and
doubling that would require nearly 2 GB of memory.  On most 32-bit
computers, 2 GB is the most memory that the operating
system will allocate to one task, so nearly nothing would be left for
the rest of Stata.

{pstd}
Why, then, is {cmd:matsize} allowed to be set so large?  Because on 64-bit
computers, such large amounts cause no difficulty.

{pstd}
For reasonable values of {cmd:matsize} (say, up to 3,200), memory consumption
is not too great.   Choose a reasonable value given the kinds of models
you fit, and remember that you can always reset the value.


{marker memory}{...}
{title:3.3  Advice on setting memory}

	    {cmd:set memory}  {it:#}[{cmd:b}|{cmd:k}|{cmd:m}|{cmd:g}] [{cmd:,} {cmdab:perm:anently}]{right:{it:#} >= 500k    }

{pin2}
	({cmd:b}, {cmd:k}, {cmd:m}, and {cmd:g} may be typed in uppercase.)


{pstd}
The advice for setting memory is the same as that for Stata/IC:  set
enough so that your datasets fit easily, but do not set so much that you
exceed the physical memory present on your computer, except in emergencies.

{pstd}
You may set memory in bytes (b), kilobytes (k), megabytes (m), or gigabytes
(g), but the number specified must be an integer, so if you want to set 1.5g,
you must set 1500m.  Actually, 1.5g is 1536m because the formulas for a kilobyte,
megabyte, and gigabyte are

		1,024 bytes = 1 KB
		1,024 KB    = 1 MB
		1,024 MB    = 1 GB

{pstd}
This detail does not matter, but this is the rule that Stata uses when
presenting numbers, so do not be surprised when 2000k is not displayed
as 2M, or 2M is displayed as 2048k.

{pstd}
If you have a 32-bit computer, the maximum amount of memory you can
set may surprise you.  Many people think that 32-bit computers can allow up to
4 GB of memory, and that in a sense is true.  Some 32-bit computers
will even allow you to install 4 GB of physical memory.  Nevertheless,
most modern operating systems will allocate a maximum of one-half the
theoretical maximum to individual tasks, meaning that most operating
systems will allow Stata only 2 GB of memory (even if they have 4
GB of memory).

{pstd}
The same one-half rule applies to 64-bit computers, but half the theoretical
limit is still 536,870,912 GB, so no one much cares.

{pstd}
This one-half limit is imposed by the operating system, not by Stata, and
the operating-system developers have good technical reasons for imposing
the rule.


{marker dta}{...}
{title:3.  Sharing .dta datasets with non-SE users}

{pstd}
You may share datasets with Stata/MP users with no changes necessary.
You may share datasets with Stata/IC or Small Stata users
as long as your dataset does not have more variables than are allowed
in those flavors of Stata.  See {help limits}.


{marker query}{...}
{title:4.  Querying memory usage}

{pstd}
The command

	    {cmd:. query memory}

{pstd}
will display the memory report that {cmd:set maxvar}, {cmd:set matsize},
and {cmd:set memory} display.  This way you do not have to change a value
to find out what you are using and how much memory is consumed by the
limits that you set.


{marker programmers}{...}
{title:5.  Advice to programmers}


{marker flavor}{...}
{title:5.1  Determining flavor}

{pstd}
Programmers can determine which flavor of Stata is running by 
examining the {help creturn} values

		                 creturn values
                        {c |} {cmd:c(flavor)   c(SE)     c(MP)}
	    {hline 12}{c +}{hline 30}
	    Small Stata {c |}  "{cmd:Small}"      0         0
	    Stata/IC    {c |}  "{cmd:IC}"         0         0
	    Stata/SE    {c |}  "{cmd:IC}"         1         0
	    Stata/MP    {c |}  "{cmd:IC}"         1         1
	    {hline 12}{c BT}{hline 30}


{marker macshift}{...}
{title:5.2  Avoid macro shift in program loops}

{pstd}
{helpb macro:macro shift} has terrible performance implications when used with
variable lists containing 20,000 or more variables.  We recommend avoiding the
use of {cmd:macro shift} in loops and instead 
using either {helpb foreach} or "double
indirection".  Double indirection means referring to {cmd:``i''} when
{cmd:`i'} contains a number 1, 2, ....


{title:Also see}

{psee}
{space 2}Help:  
{manhelp about R},
{manhelp data_types D:data types},
{manhelp matsize R},
{manhelp memory D},
{manhelp set R},
{manhelp flavors U:Flavors of Stata},
{help limits},
{help small_stata:Small Stata},
{help stataic:Stata/IC},
{help statamp:Stata/MP}
{p_end}
